/* ######################################################################### */
/* ##                                                                     ## */
/* ##  D B T U T O O L S / C O R E / C O N F I G S E R V I C E . J A V A  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Wrapper class, which handles all the configuration    ## */
/* ##               stuff.                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2002  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This library is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU Lesser General Public         ## */
/* ##  License as published by the Free Software Foundation; either       ## */
/* ##  version 2.1 of the License, or (at your option) any later version. ## */
/* ##                                                                     ## */
/* ##  This library is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  ## */
/* ##  Lesser General Public License for more details.                    ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU Lesser General Public   ## */
/* ##  License along with this library; if not, write to the Free         ## */
/* ##  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,     ## */
/* ##  MA 02111-1307 USA                                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Remark ....: <none>                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2002-11-19                                            ## */
/* ##  Changes ...: ----- ConfigService.java V00.99/R020 ---------------- ## */
/* ##               <none>                                                ## */
/* ##               ----- ConfigService.java V00.99/R011 ---------------- ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */



/* ##### Presettings */
/* ------------------------------------------------------------------------- */
package org.dbtutools.DbtuTools.core;
/* ------------------------------------------------------------------------- */





/* ##### Class definition */
/** ************************************************************************
 * One of the central services offered to all the DbtuTools tools
 * (of course, others can use them as well).<br />
 * <br />
 * The class works as a wrapper, covering the configuration stuff and
 * hiding the different Config Sources and their types. Currently, Unix
 * configuration files located in the /opt/dbtutools tree
 * [<a target="_parent" href="../../../../../pages/appendixA/optree.unix.html">UNIX</a>][<a target="_parent" href="../../../../../pages/appendixA/optree.win32.html">Wintel</a>]
 * are the only Config Source type, but other configuration file types
 * like Windows INI, XML etc. as well as directory services like LDAP
 * might be an option too.<br />
 * <br />
 * More information about the Unix configuration file format can be
 * found in the detailed description of the
 * {@link org.edelbyte.util.UnixCfgParser} class.
 *
 * @author Daniel Scheibli &lt;daniel.scheibli at edelbyte.org&gt;
 * ************************************************************************* */
public class ConfigService
{
 /* ##### Presettings (class fields) */
 /* ------------------------------------------------------------------------ */
 /** ***********************************************************************
  * Constant standing for an unknown Config Source.
  * ************************************************************************ */
 public static final int CONFIG_SOURCE_UNKNOWN = -1; 
 /** ***********************************************************************
  * Constant standing for a Unix Configuration files as Config Source.
  * ************************************************************************ */
 public static final int CONFIG_SOURCE_UNIXCFG = 0; 
 /* ------------------------------------------------------------------------ */ 





 /* ##### Presettings (instance fields)
 /* ------------------------------------------------------------------------ */
 /** ***********************************************************************
  * Field holding the given DB2 Instance name.
  * ************************************************************************ */
 protected java.lang.String     oInstanceName       = null;
 /** ***********************************************************************
  * Field holding the given DB2 Database name(s).
  * ************************************************************************ */
 protected java.util.Hashtable  oDatabaseNames      = new java.util.Hashtable();
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 /** ***********************************************************************
  * Field holding the Instance level properties read from the Config Source.
  * ************************************************************************ */
 protected java.util.Properties oInstanceProperties = new java.util.Properties(); 
 /** ***********************************************************************
  * Field holding the Database level properties read from the Config Source.
  * ************************************************************************ */
 protected java.util.Hashtable  oDatabaseProperties = new java.util.Hashtable();  
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 /** ***********************************************************************
  * Field holding the machine wide /opt/dbtutools tree root.
  * ************************************************************************ */
 protected java.lang.String     oOptTreeRoot        = null;
 /** ***********************************************************************
  * Field holding the path for the DB2 Instance configuration files
  * within the /opt/dbtutools tree.
  * ************************************************************************ */
 protected java.lang.String     oOptTreeInstanceCfg = null;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 /** ***********************************************************************
  * Field holding the Config Source type.
  * ************************************************************************ */
 protected int                  iConfigSourceType   = CONFIG_SOURCE_UNKNOWN;
 /* ------------------------------------------------------------------------ */





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   C O N F I G S E R V I C E ()                                     ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Creates a new instance of this class which addresses the current
  * DB2 Instance, but no DB2 database(s). During the instantiation,
  * the {@link #refresh(boolean, java.lang.String[])} method is called
  * implicitly.
  *   
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the name of the current DB2 Instance
  *            could not be detected. If the /opt/dbtutools tree could
  *            not be located, the exception is thrown as well.
  * ************************************************************************ */
 public ConfigService( ) throws org.edelbyte.util.UnknownValueException
 { this( null, new java.lang.String[] {} ); }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   C O N F I G S E R V I C E ()                                     ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Creates a new instance of this class which addresses the given
  * DB2 Instance, but no DB2 database(s). During the instantiation,
  * the {@link #refresh(boolean, java.lang.String[])} method is called
  * implicitly.
  *   
  * @param oInstanceName
  *        The DB2 Instance to be addressed. If <tt>null</tt> is
  *        given, the current DB2 Instance will be used.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the name of the current DB2 Instance
  *            could not be detected. If the /opt/dbtutools tree could
  *            not be located, the exception is thrown as well.
  * ************************************************************************ */
 public ConfigService( java.lang.String oInstanceName ) throws org.edelbyte.util.UnknownValueException
 { this( oInstanceName, new java.lang.String[] {} ); }
 


 
 
 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   C O N F I G S E R V I C E ()                                     ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Creates a new instance of this class which addresses the given
  * DB2 Instance, as well as the given DB2 database. During the
  * instantiation, the {@link #refresh(boolean, java.lang.String[])}
  * method is called implicitly.
  *   
  * @param oInstanceName
  *        The DB2 Instance to be addressed. If <tt>null</tt> is
  *        given, the current DB2 Instance will be used.
  * @param oDatabaseName
  *        The DB2 Database to be addressed. If <tt>null</tt> is
  *        given, no DB2 Database will be used.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the name of the current DB2 Instance
  *            could not be detected. If the /opt/dbtutools tree could
  *            not be located, the exception is thrown as well.
  * ************************************************************************ */
 public ConfigService( java.lang.String oInstanceName,
		       java.lang.String oDatabaseName ) throws org.edelbyte.util.UnknownValueException
 { this( oInstanceName, new java.lang.String[] { oDatabaseName } ); }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   C O N F I G S E R V I C E ()                                     ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Creates a new instance of this class which addresses the given
  * DB2 Instance, as well as the given DB2 database(s). During the
  * instantiation, the {@link #refresh(boolean, java.lang.String[])}
  * method is called implicitly.
  *   
  * @param oInstanceName
  *        The DB2 Instance to be addressed. If <tt>null</tt> is
  *        given, the current DB2 Instance will be used.
  * @param oDatabaseNames
  *        The DB2 Database(s) to be addressed. If <tt>null</tt> is
  *        given, no DB2 Databases will be used.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the name of the current DB2 Instance
  *            could not be detected. If the /opt/dbtutools tree could
  *            not be located, the exception is thrown as well.
  * ************************************************************************ */
 public ConfigService( java.lang.String   oInstanceName,
		       java.lang.String[] oDatabaseNames ) throws org.edelbyte.util.UnknownValueException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  java.util.HashMap oOptTreePaths;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */  
  int               I;
  /* ----------------------------------------------------------------------- */



  /* ##### Check the given parameters */
  /* ----------------------------------------------------------------------- */  
  if( oInstanceName == null )
  { this.oInstanceName = org.dbtutools.DbtuTools.core.db2.Instance.getInstance().getInstanceName(); }
  else
  { this.oInstanceName = oInstanceName; }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( oDatabaseNames == null )
  { oDatabaseNames = new java.lang.String[] {}; }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  for( I = 0; I < oDatabaseNames.length; I++ )
  {
   if( oDatabaseNames[I] == null )
   { throw new java.lang.NullPointerException(); }
   oDatabaseNames[I] = oDatabaseNames[I].toLowerCase();
   this.oDatabaseNames.put(      oDatabaseNames[I], new java.lang.Integer(1)   );
   this.oDatabaseProperties.put( oDatabaseNames[I], new java.util.Properties() );
  }
  /* ----------------------------------------------------------------------- */



  /* ##### Detect the /opt/dbtutools tree root */
  /* ##### (once per object lifetime is enough) */
  /* ----------------------------------------------------------------------- */
  oOptTreePaths = org.dbtutools.DbtuTools.core.OptDbtuToolsTree.getInstance().getPathNames();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  this.oOptTreeRoot        = oOptTreePaths.get("Tree.Root").toString();
  this.oOptTreeInstanceCfg = oOptTreePaths.get("Tree.Instance.Cfg").toString();
  /* ----------------------------------------------------------------------- */



  /* ##### Fill the configuration buffers */
  /* ----------------------------------------------------------------------- */
  this.refresh( true, oDatabaseNames );
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   G E T C O N F I G S O U R C E S E T T I N G S ()                 ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * This method determines how to find and read the Config Sources.
  * Therefore it reads the dbtuswitch.cfg file
  * [<a target="_parent" href="../../../../../pages/chapter03/dbtuswitch.unix.html">UNIX</a>][<a target="_parent" href="../../../../../pages/chapter03/dbtuswitch.win32.html">Wintel</a>]
  * and stores the information in the object fields.<br />
  * <br />
  * The method is called by the
  * {@link #refresh(boolean, java.lang.String[])}
  * method.
  *   
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the dbtuswitch.cfg could not be found
  *            or read. If there is no entry for the DB2 instance
  *            (addressed by this object), the exception is thrown as well.
  * ************************************************************************ */
 protected void getConfigSourceSettings( ) throws org.edelbyte.util.UnknownValueException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  java.io.BufferedReader oCfgFileReader = null;
  java.lang.String       oCurrentRow    = null;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  java.lang.String       oSearchString  = ("Instance=" + this.oInstanceName + ":ConfigSource=");
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  java.lang.String       oDummy;
  /* ----------------------------------------------------------------------- */
  


  /* ##### Detect the Config Source type */
  /* ##### (reading the dbtuswitch.cfg file) */
  /* ----------------------------------------------------------------------- */
  try
  {
   oCfgFileReader = new java.io.BufferedReader( new java.io.FileReader(oOptTreeRoot + java.io.File.separator + "dbtuswitch.cfg") );
   while( (oCurrentRow = oCfgFileReader.readLine()) != null )
   {
    /* ##### If row containing the Config Source settings is found,... */
    if( oCurrentRow.startsWith(oSearchString) )
    {
     /* ##### Extract the value part */
     oDummy = oCurrentRow.substring( oSearchString.length() ).trim();
     /* ##### If Config Source type is set to Unix Configuration files */
     /* ##### (currently there are no other options like LDAP etc.) */
     if( oDummy.compareToIgnoreCase( "UnixCfg" ) == 0 )
     { this.iConfigSourceType = CONFIG_SOURCE_UNIXCFG; }
    }
   }
   oCfgFileReader.close();
  }
  catch( java.io.FileNotFoundException oException )
  { throw new org.edelbyte.util.UnknownValueException( "The dbtuswitch.cfg file could not be found." ); }
  catch( java.io.IOException oException )
  { throw new org.edelbyte.util.UnknownValueException( "The dbtuswitch.cfg file could not be read." ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( iConfigSourceType == CONFIG_SOURCE_UNKNOWN )
  { throw new org.edelbyte.util.UnknownValueException( "The dbtuswitch.cfg file does not contain an ConfigSource entry for DB2 Instance \"" + this.oInstanceName + "\"." ); }
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   G E T P R O P E R T Y ()                                         ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Searches the different properties fields for the specified property
  * key.
  *   
  * @param oLevels
  *        Defines the levels to search, as well as the search order.
  *        <code>&quot;I&quot;</code> stands for the DB2 Instance level
  *        and <code>&quot;D&quot;</code> stands for the DB2 Database
  *        level. Of course you can combine them to
  *        <code>&quot;DI&quot;</code> or <code>&quot;ID&quot;</code>.
  *        Please notice that every defined level is searched whether
  *        there was a match in before or not. So the last match wins.
  * @param oDatabaseName
  *        The name of the DB2 Database to search the properties.
  *        If <tt>null</tt> is given, the DB2 Instance properties
  *        are searched only.
  * @param oKey
  *        The properties key (ignoring case considerations).
  * @exception java.lang.IllegalArgumentException
  *            For the case there is an invalid parameter.
  * @exception java.lang.NullPointerException
  *            If one of the arguments was <code>null</code>.
  * ************************************************************************ */
 public java.lang.String getProperty( java.lang.String oLevels,
                                      java.lang.String oDatabaseName,
				      java.lang.String oKey ) throws java.lang.IllegalArgumentException,
 				                                     java.lang.NullPointerException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  char[]           acLevelsOrder;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  java.lang.String oValue = null;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int    I;
  /* ----------------------------------------------------------------------- */



  /* ##### Check and process the given arguments */
  /* ----------------------------------------------------------------------- */
  if( oLevels == null )
  { throw new java.lang.NullPointerException(); } 
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oLevels = oLevels.trim().toUpperCase();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( (oLevels.compareTo("ID") != 0) &&
      (oLevels.compareTo("DI") != 0) &&
      (oLevels.compareTo("I")  != 0) &&
      (oLevels.compareTo("D")  != 0) )
  { throw new java.lang.IllegalArgumentException( "Level priority indicator contains no valid entry." ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( oDatabaseName != null )
  {
   if( this.oDatabaseNames.get( oDatabaseName ) == null )
   { throw new java.lang.IllegalArgumentException( "Only database names which has been given during object initialization are allowed." ); }
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( oKey == null )
  { throw new java.lang.NullPointerException(); } 
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oKey = oKey.trim().toUpperCase();
  /* ----------------------------------------------------------------------- */
  


  /* ##### Walk through the different indicated levels  */
  /* ----------------------------------------------------------------------- */
  acLevelsOrder = oLevels.toCharArray();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  for( I = 0; I < acLevelsOrder.length; I++ )
  {
   switch( acLevelsOrder[I] )
   {
    /* ##### For Instance level,... */
    case 'I':
         if( oInstanceProperties.containsKey( oKey ) )
	 { oValue = oInstanceProperties.get( oKey ).toString(); }
         break;
    /* ##### For Database level,... */
    case 'D':
         if( oDatabaseName == null )
         { continue; } 
         if( ((java.util.Hashtable) oDatabaseProperties.get( oDatabaseName )).containsKey( oKey ) )
	 { oValue = ((java.util.Hashtable) oDatabaseProperties.get( oDatabaseName )).get( oKey ).toString(); }
         break;
   }
  }
  /* ----------------------------------------------------------------------- */



  /* ##### Return the value */
  /* ----------------------------------------------------------------------- */
  return( oValue );
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   G E T P R O P E R T Y ()                                         ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Searches the different properties fields for the specified property
  * key.
  *   
  * @param oLevels
  *        Defines the levels to search, as well as the search order.
  *        <code>&quot;I&quot;</code> stands for the DB2 Instance level
  *        and <code>&quot;D&quot;</code> stands for the DB2 Database
  *        level. Of course you can combine them to
  *        <code>&quot;DI&quot;</code> or <code>&quot;ID&quot;</code>.
  *        Please notice that every defined level is searched whether
  *        there was a match in before or not. So the last match wins.
  * @param oDatabaseName
  *        The name of the DB2 Database to search the properties.
  *        If <tt>null</tt> is given, the DB2 Instance properties
  *        are searched only.
  * @param oKey
  *        The properties key (ignoring case considerations).
  * @param oDefaultKey
  *        For the case, that there was no match, this default value
  *        is returned.
  * @exception java.lang.IllegalArgumentException
  *            For the case there is an invalid parameter.
  * @exception java.lang.NullPointerException
  *            If one of the arguments was <code>null</code>.
  * ************************************************************************ */
 public java.lang.String getProperty( java.lang.String oLevels,
                                      java.lang.String oDatabaseName,
				      java.lang.String oKey,
				      java.lang.String oDefaultValue ) throws java.lang.IllegalArgumentException,
 				                                              java.lang.NullPointerException
 {
  /* ##### Get value and apply default (if there was no value) */
  /* ----------------------------------------------------------------------- */
  java.lang.String oValue = this.getProperty( oLevels, oDatabaseName, oKey );
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  if( oValue == null )
  { return( oDefaultValue ); }
  else
  { return( oValue ); }
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H ()                                                 ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * As the name says, the method (re)reads the Config Sources and stores
  * all found properties in the corresponding object fields.<br />
  * <br /> 
  * Please notice, that this method covers the Config Source for the
  * DB2 Instance only.
  *   
  * @param bInstanceFlag
  *        Indicates whether or not the properties for the DB2 Instance
  *        should be (re)read. When called with <code>false</code>, this
  *        method has nothing to do.
  * @exception java.lang.IllegalArgumentException
  *            For the case, that there is an invalid parameter.
  * @exception java.lang.NullPointerException
  *            If one of the arguments was <code>null</code>.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 public void refresh( boolean bInstanceFlag ) throws  java.lang.IllegalArgumentException,
 				                      java.lang.NullPointerException,
						      org.edelbyte.util.UnknownValueException
 { this.refresh( bInstanceFlag, new java.lang.String[] {} ); }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H ()                                                 ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * As the name says, the method (re)reads the Config Sources and stores
  * all found properties in the corresponding object fields.
  *   
  * @param bInstanceFlag
  *        Indicates whether or not the properties for the DB2 Instance
  *        should be (re)read.
  * @param oDatabaseName
  *        The name of the DB2 Database for which the properties
  *        should be (re)read. If <tt>null</tt> is given, no
  *        DB2 Databases are taken into account.
  * @exception java.lang.IllegalArgumentException
  *            For the case, that there is an invalid parameter.
  * @exception java.lang.NullPointerException
  *            If one of the arguments was <code>null</code>.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 public void refresh( boolean          bInstanceFlag,
		      java.lang.String oDatabaseName ) throws java.lang.IllegalArgumentException,
 				                              java.lang.NullPointerException,
						              org.edelbyte.util.UnknownValueException
 { this.refresh( bInstanceFlag, new java.lang.String[] { oDatabaseName } ); }




 
 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H ()                                                 ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * As the name says, the method (re)reads the Config Sources and stores
  * all found properties in the corresponding object fields.
  *   
  * @param bInstanceFlag
  *        Indicates whether or not the properties for the DB2 Instance
  *        should be (re)read.
  * @param oDatabaseNames
  *        The name of the DB2 Database(s) for which the properties
  *        should be (re)read. If <tt>null</tt> is given, no DB2
  *        Databases are taken into account.
  * @exception java.lang.IllegalArgumentException
  *            For the case, that there is an invalid parameter.
  * @exception java.lang.NullPointerException
  *            If one of the arguments was <code>null</code>.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 public void refresh( boolean            bInstanceFlag,
		      java.lang.String[] oDatabaseNames ) throws java.lang.IllegalArgumentException,
 				                                 java.lang.NullPointerException,
						                 org.edelbyte.util.UnknownValueException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  int I;
  /* ----------------------------------------------------------------------- */



  /* ##### Check the given arguments */
  /* ----------------------------------------------------------------------- */
  if( oDatabaseNames == null )
  { oDatabaseNames = new java.lang.String[] {}; }
  /* ----------------------------------------------------------------------- */



  /* ##### (Re)read the Config Source settings */
  /* ----------------------------------------------------------------------- */
  this.getConfigSourceSettings();
  /* ----------------------------------------------------------------------- */



  /* ##### (Re)read the different properties for the Config Source */
  /* ----------------------------------------------------------------------- */
  if( bInstanceFlag == true )
  { this.refreshInstanceProperties(); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  for( I = 0; I < oDatabaseNames.length; I++ )
  {
   if( oDatabaseNames[I] == null )
   { throw new java.lang.NullPointerException(); }
   oDatabaseNames[I] = oDatabaseNames[I].toLowerCase();
   if( this.oDatabaseNames.get( oDatabaseNames[I] ) == null )
   { throw new java.lang.IllegalArgumentException( "Only database names which has been given during object initialization are allowed." ); }
   this.refreshDatabaseProperties( oDatabaseNames[I] );
  }
  /* ----------------------------------------------------------------------- */
 }
 




 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H A L L ()                                           ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * As the name says, the method (re)reads all Config Sources and stores
  * all found properties in the corresponding object fields. The method
  * internally calls the {@link #refresh(boolean, java.lang.String[])}
  * method.
  *   
  * @exception java.lang.IllegalArgumentException
  *            For the case, that there is an invalid parameter.
  * @exception java.lang.NullPointerException
  *            If one of the arguments was <code>null</code>.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 public void refreshAll( ) throws java.lang.IllegalArgumentException,
 				  java.lang.NullPointerException,
				  org.edelbyte.util.UnknownValueException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */ 
  java.util.Enumeration oEnum       = this.oDatabaseNames.keys();
  int                   iEnumLength = this.oDatabaseNames.size();
  java.lang.String      aoDummy[]   = new java.lang.String[iEnumLength];
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int                   I;
  /* ----------------------------------------------------------------------- */  


  
  /* ##### Copy the Hashtable into an String array */
  /* ##### REMARK: If one day the oDatabaseNames Hashtable will be, */
  /* #####         manipulated during lifetime, this function needs */
  /* #####         to be changed. */  
  /* ----------------------------------------------------------------------- */
  for( I = 0; I < iEnumLength; I++ )
  { aoDummy[I] = oEnum.nextElement().toString(); }
  /* ----------------------------------------------------------------------- */



  /* ##### Call the original refresh() method */
  /* ----------------------------------------------------------------------- */  
  this.refresh( true, aoDummy );
  /* ----------------------------------------------------------------------- */  
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H D A T A B A S E P R O P E R T I E S ()             ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Submethod which is called by the
  * {@link #refresh(boolean, java.lang.String[])} method to
  * (re)read the Config Sources for the given DB2 Database.
  *   
  * @param oDatabaseName
  *        The name of the DB2 Database for which the properties
  *        should be (re)read.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 private void refreshDatabaseProperties( java.lang.String oDatabaseName ) throws org.edelbyte.util.UnknownValueException
 {
  /* ##### Do the Config Source specific code */
  /* ----------------------------------------------------------------------- */
  if( this.iConfigSourceType == CONFIG_SOURCE_UNIXCFG )
  { this.refreshDatabasePropertiesUnixCfg( oDatabaseName ); }
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H D A T A B A S E P R O P ... U N I X C F G ()       ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Subsubmethod for (re)reading the Unix configuration file for
  * the given DB2 Database.
  *   
  * @param oDatabaseName
  *        The name of the DB2 Database for which the properties
  *        should be (re)read.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 private void refreshDatabasePropertiesUnixCfg( java.lang.String oDatabaseName ) throws org.edelbyte.util.UnknownValueException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */  
  org.edelbyte.util.UnixCfgParser oParser     = new org.edelbyte.util.UnixCfgParser();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */  
  java.util.Properties            oProperties = null;
  java.lang.String                oFileName   = this.oOptTreeInstanceCfg + java.io.File.separator + "db." + oDatabaseName + ".cfg";
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oParser.setBetweenCheckEnabled( true );
  /* ----------------------------------------------------------------------- */
  


  /* ##### Call the UnixCfg parser and store the results */
  /* ----------------------------------------------------------------------- */
  try
  { oProperties = oParser.parse( new java.io.BufferedReader(new java.io.FileReader(oFileName)) ); }
  catch( java.lang.Exception oException )
  { throw new org.edelbyte.util.UnknownValueException( "UnixCfgParser: Parse of " + oFileName + " failed: " + oException.getMessage() ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oDatabaseProperties.put( oDatabaseName, oProperties );
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H I N S T A N C E P R O P E R T I E S ()             ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Submethod which is called by the
  * {@link #refresh(boolean, java.lang.String[])} method to
  * (re)read the Config Sources for the addressed DB2 Instance.
  *   
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 private void refreshInstanceProperties( ) throws org.edelbyte.util.UnknownValueException
 {
  /* ##### Do the Config Source specific code */
  /* ----------------------------------------------------------------------- */
  if( this.iConfigSourceType == CONFIG_SOURCE_UNIXCFG )
  { this.refreshInstancePropertiesUnixCfg(); }
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   R E F R E S H I N S T A N C E P R O P ... U N I X C F G ()       ## */
 /* ##                                                                    ## */
 /* ######################################################################## */
 /** ***********************************************************************
  * Subsubmethod for (re)reading the Unix configuration file for
  * the addressed DB2 Instance.
  *   
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the Config Source could not be read.
  * ************************************************************************ */
 private void refreshInstancePropertiesUnixCfg( ) throws org.edelbyte.util.UnknownValueException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */  
  org.edelbyte.util.UnixCfgParser oParser     = new org.edelbyte.util.UnixCfgParser();
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */  
  java.util.Properties            oProperties = null;
  java.lang.String                oFileName   = oOptTreeInstanceCfg + java.io.File.separator + "dbm.cfg";
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oParser.setBetweenCheckEnabled( true );
  /* ----------------------------------------------------------------------- */
  


  /* ##### Call the UnixCfg parser and store the results */
  /* ----------------------------------------------------------------------- */
  try
  { oProperties = oParser.parse( new java.io.BufferedReader(new java.io.FileReader(oFileName)) ); }
  catch( java.lang.Exception oException )
  { throw new org.edelbyte.util.UnknownValueException( "UnixCfgParser: Parse of " + oFileName + " failed: " + oException.getMessage() ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  oInstanceProperties = oProperties;
  /* ----------------------------------------------------------------------- */
 }
}
/* ------------------------------------------------------------------------- */




